/*
 * zeroII_uart.c
 *
 *  Created on: May 30, 2022
 *      Author: sichk
 */

#include "stm32f1xx_hal.h"
#include "zeroII_uart.h"
#include "float.h"

typedef enum
{
	ZEROII_GET_STATUS = 0x5A,
	ZEROII_SET_SYSTEM_Z0 = 0xF2,
	ZEROII_GET_SYSTEM_Z0 = 0xC4,
	ZEROII_SET_FQ_GET_RX = 0x6D,
	ZEROII_SET_FQ_GET_RXSWRRL = 0xA3,
	ZEROII_GET_RX_DATA = 0x7C,
	ZEROII_GET_RX_SWR_RL = 0x9A,
	ZEROII_GET_FW_VERSION = 0xE5,
} ZeroII_Commands_e;

typedef enum {
	ZEROII_STATUS_BUSY_USB = 1,
	ZEROII_STATUS_BUSY_SPI,
	ZEROII_STATUS_BUSY_I2C,
	ZEROII_STATUS_BUSY_UART,
	ZEROII_STATUS_IDLE,
	ZEROII_STATUS_READY,
	ZEROII_STATUS_ERROR,
} ZeroII_Status_e;

//ZEROII_DEV_INFO_t zeroII_dev_info = {0};
//
//ZEROII_RESULT_t zeroII_result = {0};

GPIO_TypeDef *zeroII_rst_GPIO_Port = 0;
uint16_t zeroII_rst_GPIO_Pin = 0;

#define zeroII_RESET_ON()		HAL_GPIO_WritePin(zeroII_rst_GPIO_Port, zeroII_rst_GPIO_Pin, GPIO_PIN_RESET)
#define zeroII_RESET_OFF()		HAL_GPIO_WritePin(zeroII_rst_GPIO_Port, zeroII_rst_GPIO_Pin, GPIO_PIN_SET)

#define UART_IFC_TX_BUF_SZ		20
#define UART_IFC_RX_BUF_SZ		20

uint8_t uart_tx_buffer[UART_IFC_TX_BUF_SZ] = { 0 };
uint8_t uart_rx_buffer[UART_IFC_RX_BUF_SZ] = { 0 };


uint32_t zeroII_operation_timestamp = 0;
uint32_t zeroII_minimal_delay = 0;
static bool zeroII_idle = false;
static bool measure_result_type = false;
static uint32_t systemZ0 = 50000;
static ZEROII_DEV_INFO_t zeroII_dev_info = {0};
static ZEROII_RESULT_t zeroII_meas_result = {0};

UART_HandleTypeDef *zero_huart;

bool zeroII_is_idle(void);

static const uint8_t crc8_table[256] = {0x00, 0x07, 0x0E, 0x09, 0x1C, 0x1B, 0x12, 0x15, 0x38, 0x3F, 0x36, 0x31, 0x24, 0x23, 0x2A, 0x2D, 0x70, 0x77, 0x7E, 0x79, 0x6C, 0x6B, 0x62, 0x65, 0x48, 0x4F, 0x46, 0x41, 0x54, 0x53, 0x5A, 0x5D, 0xE0, 0xE7, 0xEE, 0xE9, 0xFC, 0xFB, 0xF2, 0xF5, 0xD8, 0xDF, 0xD6, 0xD1, 0xC4, 0xC3, 0xCA, 0xCD, 0x90, 0x97, 0x9E, 0x99, 0x8C, 0x8B, 0x82, 0x85, 0xA8, 0xAF, 0xA6, 0xA1, 0xB4, 0xB3, 0xBA, 0xBD, 0xC7, 0xC0, 0xC9, 0xCE, 0xDB, 0xDC, 0xD5, 0xD2, 0xFF, 0xF8, 0xF1, 0xF6, 0xE3, 0xE4, 0xED, 0xEA, 0xB7, 0xB0, 0xB9, 0xBE, 0xAB, 0xAC, 0xA5, 0xA2, 0x8F, 0x88, 0x81, 0x86, 0x93, 0x94, 0x9D, 0x9A, 0x27, 0x20, 0x29, 0x2E, 0x3B, 0x3C, 0x35, 0x32, 0x1F, 0x18, 0x11, 0x16, 0x03, 0x04, 0x0D, 0x0A, 0x57, 0x50, 0x59, 0x5E, 0x4B, 0x4C, 0x45, 0x42, 0x6F, 0x68, 0x61, 0x66, 0x73, 0x74, 0x7D, 0x7A, 0x89, 0x8E, 0x87, 0x80, 0x95, 0x92, 0x9B, 0x9C, 0xB1, 0xB6, 0xBF, 0xB8, 0xAD, 0xAA, 0xA3, 0xA4, 0xF9, 0xFE, 0xF7, 0xF0, 0xE5, 0xE2, 0xEB, 0xEC, 0xC1, 0xC6, 0xCF, 0xC8, 0xDD, 0xDA, 0xD3, 0xD4, 0x69, 0x6E, 0x67, 0x60, 0x75, 0x72, 0x7B, 0x7C, 0x51, 0x56, 0x5F, 0x58, 0x4D, 0x4A, 0x43, 0x44, 0x19, 0x1E, 0x17, 0x10, 0x05, 0x02, 0x0B, 0x0C, 0x21, 0x26, 0x2F, 0x28, 0x3D, 0x3A, 0x33, 0x34, 0x4E, 0x49, 0x40, 0x47, 0x52, 0x55, 0x5C, 0x5B, 0x76, 0x71, 0x78, 0x7F, 0x6A, 0x6D, 0x64, 0x63, 0x3E, 0x39, 0x30, 0x37, 0x22, 0x25, 0x2C, 0x2B, 0x06, 0x01, 0x08, 0x0F, 0x1A, 0x1D, 0x14, 0x13, 0xAE, 0xA9, 0xA0, 0xA7, 0xB2, 0xB5, 0xBC, 0xBB, 0x96, 0x91, 0x98, 0x9F, 0x8A, 0x8D, 0x84, 0x83, 0xDE, 0xD9, 0xD0, 0xD7, 0xC2, 0xC5, 0xCC, 0xCB, 0xE6, 0xE1, 0xE8, 0xEF, 0xFA, 0xFD, 0xF4, 0xF3}; /* 8-bit table */

uint8_t crc8_buffer(const uint8_t *buf, uint32_t len)
{
	uint8_t crc8_calculation_register = 0;

    while(len-- > 0)
    {
        crc8_calculation_register = crc8_table[crc8_calculation_register ^ (*buf)];
        crc8_calculation_register &= 0xFF;
        buf++;
    }
    /* Return the CRC computed value */
    return crc8_calculation_register;
}

void pack_uart_msg(uint8_t* msg, uint32_t len)
{
	uint8_t crc8 = crc8_buffer(msg, len);
	msg += len;
	*msg = crc8;
	msg++;
	*msg = crc8 ^ 0xFF;
}

bool unpack_uart_msg(uint8_t* msg, uint32_t len)
{
	len--;
	if (crc8_buffer(msg, len) || (msg[len] != (msg[len - 1] ^ 0xFF))) return false;
	return true;
}

void zeroII_error_handler()
{
	while(1);
}

void zeroII_init(UART_HandleTypeDef *huart, GPIO_TypeDef *RESET_GPIO_Port, uint16_t RESET_GPIO_Pin)
{
	if (huart == 0) zeroII_error_handler();

	switch ((int32_t)RESET_GPIO_Port) {
#ifdef GPIOA
	case (int32_t)GPIOA:
	    __HAL_RCC_GPIOA_CLK_ENABLE();
		break;
#endif
#ifdef GPIOB
	case (int32_t)GPIOB:
	    __HAL_RCC_GPIOB_CLK_ENABLE();
		break;
#endif
#ifdef GPIOC
	case (int32_t)GPIOC:
	    __HAL_RCC_GPIOC_CLK_ENABLE();
		break;
#endif
#ifdef GPIOD
	case (int32_t)GPIOD:
	    __HAL_RCC_GPIOD_CLK_ENABLE();
		break;
#endif
#ifdef GPIOE
	case (int32_t)GPIOE:
	    __HAL_RCC_GPIOE_CLK_ENABLE();
		break;
#endif
#ifdef GPIOF
	case (int32_t)GPIOF:
	    __HAL_RCC_GPIOF_CLK_ENABLE();
		break;
#endif
#ifdef GPIOG
	case (int32_t)GPIOG:
	    __HAL_RCC_GPIOG_CLK_ENABLE();
		break;
#endif
#ifdef GPIOH
	case (int32_t)GPIOH:
	    __HAL_RCC_GPIOH_CLK_ENABLE();
		break;
#endif
#ifdef GPIOI
	case (int32_t)GPIOI:
	    __HAL_RCC_GPIOI_CLK_ENABLE();
		break;
#endif
#ifdef GPIOJ
	case (int32_t)GPIOJ:
	    __HAL_RCC_GPIOJ_CLK_ENABLE();
		break;
#endif
#ifdef GPIOK
	case (int32_t)GPIOK:
	    __HAL_RCC_GPIOK_CLK_ENABLE();
		break;
#endif
#ifdef GPIOL
	case (int32_t)GPIOL:
	    __HAL_RCC_GPIOL_CLK_ENABLE();
		break;
#endif
#ifdef GPIOM
	case (int32_t)GPIOM:
	    __HAL_RCC_GPIOM_CLK_ENABLE();
		break;
#endif
#ifdef GPION
	case (int32_t)GPION:
	    __HAL_RCC_GPION_CLK_ENABLE();
		break;
#endif
	default:
		zeroII_error_handler();
	}

	zeroII_rst_GPIO_Port = RESET_GPIO_Port;
	zeroII_rst_GPIO_Pin = RESET_GPIO_Pin;

	GPIO_InitTypeDef GPIO_InitStruct = {0};

    GPIO_InitStruct.Pin = zeroII_rst_GPIO_Pin;
    GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
    GPIO_InitStruct.Pull = GPIO_PULLUP;
    GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_HIGH;
    HAL_GPIO_Init(zeroII_rst_GPIO_Port, &GPIO_InitStruct);

    zeroII_RESET_ON();
    HAL_Delay(10);
    zeroII_RESET_OFF();
    HAL_Delay(190);

	zero_huart = huart;

	HAL_UART_Receive(zero_huart, uart_rx_buffer, UART_IFC_RX_BUF_SZ, 10); //Clean RX before start communication

	zeroII_idle = true;
	zeroII_minimal_delay = 0;
	zeroII_operation_timestamp = HAL_GetTick();
}

// Device info
const ZEROII_DEV_INFO_t* const zeroII_get_info(void)
{
	if (zeroII_idle || zeroII_is_idle())
	{
		zeroII_idle = false;

		uart_tx_buffer[0] = ZEROII_GET_FW_VERSION;
		pack_uart_msg(uart_tx_buffer, 1);
		HAL_UART_Transmit(zero_huart, uart_tx_buffer, 3, 100);
		HAL_UART_Receive(zero_huart, uart_rx_buffer, 9, 100);

		zeroII_minimal_delay = 0;
		zeroII_operation_timestamp = HAL_GetTick();

		if (unpack_uart_msg(uart_rx_buffer, 9))
		{
			zeroII_dev_info.version_maj = uart_rx_buffer[0];
			zeroII_dev_info.version_min = uart_rx_buffer[1];
			zeroII_dev_info.version_rev = uart_rx_buffer[2];
			zeroII_dev_info.zeroII_serial = *((uint32_t*) (uart_rx_buffer + 3));

			return (const ZEROII_DEV_INFO_t* const)&zeroII_dev_info;
		}
	}
	return false;
}

// SystemZ0
float zeroII_get_system_z(void)
{
	if (zeroII_idle || zeroII_is_idle())
	{
		zeroII_idle = false;

		uart_tx_buffer[0] = ZEROII_GET_SYSTEM_Z0;
		pack_uart_msg(uart_tx_buffer, 1);
		HAL_UART_Transmit(zero_huart, uart_tx_buffer, 3, 100);
		HAL_UART_Receive(zero_huart, uart_rx_buffer, 6, 100);

		zeroII_minimal_delay = 0;
		zeroII_operation_timestamp = HAL_GetTick();

		if (unpack_uart_msg(uart_rx_buffer, 6))
		{
			systemZ0 = *((uint32_t*) (uart_rx_buffer));
			return ((float)systemZ0) / 1000.0F;
		}
	}
	return false;
}

bool zeroII_set_system_z(float Z0)
{
	if ((Z0 > 0) && (zeroII_idle || zeroII_is_idle()))
	{
		zeroII_idle = false;

		systemZ0 = (uint32_t)(Z0 * 1000.0);

		uart_tx_buffer[0] = ZEROII_SET_SYSTEM_Z0;
		*((uint32_t*) (uart_tx_buffer + 1)) = systemZ0;
		pack_uart_msg(uart_tx_buffer, 5);
		HAL_UART_Transmit(zero_huart, uart_tx_buffer, 7, 100);

		zeroII_minimal_delay = 0;
		zeroII_operation_timestamp = HAL_GetTick();

		return true;
	}
	return false;
}

// Measure @ Frequency
bool zeroII_measure_RX(uint32_t fq)
{
	if ((100000 <= fq) && (fq <= 1000000000) && (zeroII_idle || zeroII_is_idle()))
	{
		zeroII_idle = false;

		uart_tx_buffer[0] = ZEROII_SET_FQ_GET_RX;
		*((uint32_t*) (uart_tx_buffer + 1)) = fq;
		pack_uart_msg(uart_tx_buffer, 5);
		HAL_UART_Transmit(zero_huart, uart_tx_buffer, 7, 100);

		measure_result_type = false;
		zeroII_minimal_delay = 50;
		zeroII_operation_timestamp = HAL_GetTick();

		return true;
	}
	return false;
}

bool zeroII_measure_RX_SWR_RL(uint32_t fq)
{
	if ((100000 <= fq) && (fq <= 1000000000) && (zeroII_idle || zeroII_is_idle()))
	{
		zeroII_idle = false;

		uart_tx_buffer[0] = ZEROII_SET_FQ_GET_RXSWRRL;
		*((uint32_t*) (uart_tx_buffer + 1)) = fq;
		pack_uart_msg(uart_tx_buffer, 5);
		HAL_UART_Transmit(zero_huart, uart_tx_buffer, 7, 100);

		measure_result_type = true;
		zeroII_minimal_delay = 50;
		zeroII_operation_timestamp = HAL_GetTick();

		return true;
	}
	return false;
}

bool zeroII_remeasure_RX(void)
{
	if (zeroII_idle || zeroII_is_idle())
	{
		zeroII_idle = false;

		uart_tx_buffer[0] = ZEROII_GET_RX_DATA;
		pack_uart_msg(uart_tx_buffer, 1);
		HAL_UART_Transmit(zero_huart, uart_tx_buffer, 3, 100);

		measure_result_type = false;
		zeroII_minimal_delay = 50;
		zeroII_operation_timestamp = HAL_GetTick();

		return true;
	}
	return false;
}

bool zeroII_remeasure_RX_SWR_RL(void)
{
	if (zeroII_idle || zeroII_is_idle())
	{
		zeroII_idle = false;

		uart_tx_buffer[0] = ZEROII_GET_RX_SWR_RL;
		pack_uart_msg(uart_tx_buffer, 1);
		HAL_UART_Transmit(zero_huart, uart_tx_buffer, 3, 100);

		measure_result_type = true;
		zeroII_minimal_delay = 50;
		zeroII_operation_timestamp = HAL_GetTick();

		return true;
	}
	return false;
}

// Check status
const ZEROII_RESULT_t* const zeroII_is_ready(void)
{
	uint32_t now = HAL_GetTick();

	if ((now - zeroII_operation_timestamp) < zeroII_minimal_delay) return false;

	uart_tx_buffer[0] = ZEROII_GET_STATUS;
	pack_uart_msg(uart_tx_buffer, 1);
	HAL_UART_Transmit(zero_huart, uart_tx_buffer, 3, 10);
	HAL_UART_Receive(zero_huart, uart_rx_buffer, 3, 10);

	if (unpack_uart_msg(uart_rx_buffer, 3) && (uart_rx_buffer[0] == ZEROII_STATUS_READY))
	{
		zeroII_minimal_delay = 0;
		zeroII_operation_timestamp = HAL_GetTick();

		if (measure_result_type)
		{
			HAL_UART_Receive(zero_huart, uart_rx_buffer, 18, 100);

			if (unpack_uart_msg(uart_rx_buffer, 18))
			{
				zeroII_meas_result.R = *((float*) (uart_rx_buffer));
				zeroII_meas_result.X = *((float*) (uart_rx_buffer + 4));
				zeroII_meas_result.SWR = *((float*) (uart_rx_buffer + 8));
				zeroII_meas_result.RL = *((float*) (uart_rx_buffer + 12));
				return (const ZEROII_RESULT_t* const)&zeroII_meas_result;
			}
			else
				return false;
		}
		else
		{
			HAL_UART_Receive(zero_huart, uart_rx_buffer, 10, 100);

			if (unpack_uart_msg(uart_rx_buffer, 10))
			{
				zeroII_meas_result.R = *((float*) (uart_rx_buffer));
				zeroII_meas_result.X = *((float*) (uart_rx_buffer + 4));
				zeroII_meas_result.SWR = 0.0;
				zeroII_meas_result.RL = 0.0;
				return (const ZEROII_RESULT_t* const)&zeroII_meas_result;
			}
			else
				return false;
		}
	}
	else
	{
		zeroII_minimal_delay = 0;
		zeroII_operation_timestamp = HAL_GetTick();
		return false;
	}
}

const ZEROII_RESULT_t* const zeroII_wait_until_ready(uint32_t timeout)
{
	uint32_t now = HAL_GetTick();

	if ((now - zeroII_operation_timestamp) < zeroII_minimal_delay)
		HAL_Delay(zeroII_minimal_delay - (now - zeroII_operation_timestamp));

	zeroII_minimal_delay = 10;

	while (timeout >= zeroII_minimal_delay)
	{
		uart_tx_buffer[0] = ZEROII_GET_STATUS;
		pack_uart_msg(uart_tx_buffer, 1);
		HAL_UART_Transmit(zero_huart, uart_tx_buffer, 3, 10);
		HAL_UART_Receive(zero_huart, uart_rx_buffer, 3, 10);

		if (unpack_uart_msg(uart_rx_buffer, 3) && (uart_rx_buffer[0] == ZEROII_STATUS_READY))
		{
			zeroII_minimal_delay = 0;
			zeroII_operation_timestamp = HAL_GetTick();

			if (measure_result_type)
			{
				HAL_UART_Receive(zero_huart, uart_rx_buffer, 18, 100);

				if (unpack_uart_msg(uart_rx_buffer, 18))
				{
					zeroII_meas_result.R = *((float*) (uart_rx_buffer));
					zeroII_meas_result.X = *((float*) (uart_rx_buffer + 4));
					zeroII_meas_result.SWR = *((float*) (uart_rx_buffer + 8));
					zeroII_meas_result.RL = *((float*) (uart_rx_buffer + 12));
					return (const ZEROII_RESULT_t* const)&zeroII_meas_result;
				}
				else
					return false;
			}
			else
			{
				HAL_UART_Receive(zero_huart, uart_rx_buffer, 10, 100);

				if (unpack_uart_msg(uart_rx_buffer, 10))
				{
					zeroII_meas_result.R = *((float*) (uart_rx_buffer));
					zeroII_meas_result.X = *((float*) (uart_rx_buffer + 4));
					zeroII_meas_result.SWR = 0.0;
					zeroII_meas_result.RL = 0.0;
					return (const ZEROII_RESULT_t* const)&zeroII_meas_result;
				}
				else
					return false;
			}
		}

		HAL_Delay(zeroII_minimal_delay);
		timeout -= zeroII_minimal_delay;
	}
	zeroII_minimal_delay = 0;
	zeroII_operation_timestamp = HAL_GetTick();
	return false;
}


bool zeroII_is_idle(void)
{
	uint32_t now = HAL_GetTick();

	if ((now - zeroII_operation_timestamp) < zeroII_minimal_delay) return false;

	uart_tx_buffer[0] = ZEROII_GET_STATUS;
	pack_uart_msg(uart_tx_buffer, 1);
	HAL_UART_Transmit(zero_huart, uart_tx_buffer, 3, 10);
	HAL_UART_Receive(zero_huart, uart_rx_buffer, 3, 10);

	zeroII_minimal_delay = 0;
	zeroII_operation_timestamp = HAL_GetTick();

	if (unpack_uart_msg(uart_rx_buffer, 3) && (uart_rx_buffer[0] == ZEROII_STATUS_IDLE))
	{
		zeroII_idle = true;
		return true;
	}
	else
		return false;
}
