/*
 * zeroII_uart.h
 *
 *  Created on: May 30, 2022
 *      Author: sichk
 */

#ifndef ZEROII_ZEROII_UART_H_
#define ZEROII_ZEROII_UART_H_

typedef enum {
    false = 0,
    true
} bool;

typedef struct {
	uint8_t version_maj;
	uint8_t version_min;
	uint8_t version_rev;
	uint32_t zeroII_serial;
} ZEROII_DEV_INFO_t;

typedef struct {
	float R;
	float X;
	float SWR;
	float RL;
} ZEROII_RESULT_t;

// Initialize GPIO for reset ZeroII, perform reset, clean uart RX etc.
void zeroII_init(UART_HandleTypeDef *huart, GPIO_TypeDef *RESET_GPIO_Port, uint16_t RESET_GPIO_Pin);

// Device info
const ZEROII_DEV_INFO_t* const zeroII_get_info(void);	// Get device FW\HW version and serial number.
														// Return handle of device info structure or 0 in case of
														// ongoing measurement in progress or I\O error.

// SystemZ0
float zeroII_get_system_z(void);				// Get system Z0 from device.
												// Return Z0 value or 0 in case of ongoing measurement in progress
												// or I\O error.

bool zeroII_set_system_z(float Z0);				// Set system Z0 to device.
												// Return true in success, false in case of ongoing measurement in progress
												// or I\O error.

// Measure @ Frequency
bool zeroII_measure_RX(uint32_t fq); 			// Perform measurements at fq frequency in short result mode.
												// Return true in success, false in case of ongoing measurement in progress
												// or fq out of range.

bool zeroII_measure_RX_SWR_RL(uint32_t fq);		// Perform measurements at fq frequency in full result mode.
												// Return true in success, false in case of ongoing measurement in progress
												// or I\O error.

bool zeroII_remeasure_RX(void);					// Perform measurements at previous frequency in short result mode.
												// Return true in success, false in case of ongoing measurement in progress
												//  or fq out of range.

bool zeroII_remeasure_RX_SWR_RL(void);			// Perform measurements at previous frequency in full result mode.
												// Return true in success, false in case of ongoing measurement in progress
												//  or fq out of range.

// Check status
const ZEROII_RESULT_t* const zeroII_is_ready(void);						// Check if result of measurements ready.
																		// Return handle of result structure or
																		// 0 if not ready yet or I\O error.

const ZEROII_RESULT_t* const zeroII_wait_until_ready(uint32_t timeout); // Wait until result of measurements ready or timeout.
																		// Return handle of result structure or 0 if not ready
																		// when timeout occurred.

#endif /* ZEROII_ZEROII_UART_H_ */
